# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or 
# redistribute it subject to the terms and conditions of the GNU General 
# Public License v.2.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
# DeviceDetector is an abstract base class for classes using various
# means to detect hardware devices, such as HAL, or the /proc filesystem.


from rhcert.tags import Constants

class DeviceDetector:

    def __init__(self, name):
        self.name = name # the simple name
        self.debug = Constants.off

    def getName(self): return self.name

    def getDevice(self, id):
        pass
        """ return an rhcert.Device with the given unique ID """

    def getDevices(self):
        pass
        """ return a list of all rhcert.Devices detected """

    def dumpDevice(self,id):
        """Return a string containing device properties for the given device"""
        pass

    def dump(self):
        """Dump device properties for all devices"""
        pass

    def getUSBBusAndPort(self, device):
        """ return (busNumber, portNumber, deviceID) """
        pass

    def getPCIBusAndID(self, device):
        """ return (domAndBus, devAndFn, deviceID) """
        pass

    def getProduct(self, device):
        return None
