# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Authors: Greg Nichols
#          Ben Rose
#

from rhcert.tags import Constants
from rhcert.suites.hwcert.hwcert.networkTest import NetworkTest
from rhcert.suites.hwcert.hwcert.wireless import Wireless

class EthernetTest(NetworkTest):
    def __init__(self, path):
        path = "Ethernet/" + path
        NetworkTest.__init__(self, path)
        self.enforceSpeed = False
        self.interfaceSpeed = 0 # speed in Mb/sec as a default.
        self.serverMode = True # server-end defined in NetworkTest, enabled here only

    def plan(self, planner):
        tests = list()
        devices = self.getNetworkDevices(planner)
        for (logicalDevice, device) in devices.items():
            if self.checkInterfaceSpeed(logicalDevice) and self.checkWired(logicalDevice):
                test = self.makeCopy()
                test.setLogicalDeviceName(logicalDevice)
                if self.getDebug() != Constants.off:
                    print("Planned %s" % test.getPath())
                test.setDevice(device)
                tests.append(test)
        return tests

    def checkInterfaceSpeed(self, logicalDevice):
        """ return True if speed matches this test """
        interfaceSpeed = self.getInterfaceSpeed(logicalDevice)
        if self.getDebug() != Constants.off:
            print("Interface speed for %s is %s vs. test %s" % (logicalDevice, interfaceSpeed, self.interfaceSpeed))
        """ only plan this test if interface speed matches """
        return (interfaceSpeed == self.interfaceSpeed)

    def checkWired(self, logicalDevice):
        if logicalDevice in Wireless().getInterfaces():
            return False
        return True
