# Copyright (c) 2012 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or 
# redistribute it subject to the terms and conditions of the GNU General 
# Public License v.2.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols


from rhcert.model.documentbase import DocumentBase
from rhcert.tags import Tags, Attributes

class KernelInfo(DocumentBase):

    def __init__(self, file):
        DocumentBase.__init__(self)
        try:
            self.load(file)
            self.element = self.document.documentElement
        except IOError:
            print("Error: could not open file " + file)

    def getProduct(self, product):
        # first check for complete match!
        product = product.upper()
        for productElement in self.getChildElements(Tags.product):
            elementProductName = productElement.getAttribute(Attributes.name).upper()
            if product == elementProductName:
                return productElement
        for productElement in self.getChildElements(Tags.product):
            elementProductName = productElement.getAttribute(Attributes.name).upper()
            # allow partial match from left
            try:
                if product.index(elementProductName) == 0:
                    return productElement
            except ValueError:
                pass
        return None

    def getRelease(self, product, release):
        productElement = self.getProduct(product)
        if productElement:
            for releaseElement in productElement.childNodes:
                if releaseElement.nodeName == Tags.release and releaseElement.getAttribute(Attributes.number) == str(release):
                    return releaseElement
        return None

    def getGAKernels(self, product, release, update):
        releaseElement = self.getRelease(product, release)
        if releaseElement:
            updates = list()
            for element in releaseElement.childNodes:
                if element.nodeName == Tags.update and element.getAttribute(Attributes.number) == str(update):
                    updates.append(element.getAttribute(Attributes.kernel))
            return updates
        # otherwise
        return None

if __name__ == "__main__":

    info = KernelInfo("kernelInfo.xml")
    for (product, release) in [("Red Hat Enterprise Linux", 5), ("Red Hat Enterprise Linux", 6), ("Red Hat MRG Realtime Linux", 2)]:
        update = 0
        while True:
            kernel = info.getGAKernel(product, release, update)
            if not kernel:
                break
            print("%s %s.%s: %s" % (product, release, update, kernel))
            update += 1


