# Copyright (c) 2017 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Benjamin Berg
#
import subprocess, re
from rhcert.tags import Constants
from rhcert.command import Command, HwCertCommandException
from rhcert.redhatrelease import RedHatRelease

def sleep_can_rtcwake():
    # Returned cached result if available
    global _have_rtcwake
    try:
        return _have_rtcwake
    except NameError:
        pass

    release = RedHatRelease()
    # RTC wakeup exists on x86 platforms (TODO: what about aarch64?)
    rtcwake_archs = [Constants.x86_64, Constants.i386, Constants.i686]

    if release.getArch() in rtcwake_archs:
        _have_rtcwake = True
    else:
        _have_rtcwake = False

    return _have_rtcwake

def sleep_rpms():
    if sleep_can_rtcwake():
        return ['util-linux']
    else:
        return []

def sleep(sleeptype, time_s=None):
    # This function must not return before wakeup.
    # If time_s is given, sleep only for the requested period of time
    # (using the real time clock to wake up).

    if time_s is None or time_s < 0:
        # Use systemd or something else?
        open('/sys/power/state', 'w').write(sleeptype)
        return True

    assert sleep_can_rtcwake(), "Cannot sleep for a specific time on this architecture."

    # We use rtcwake here. In theory we could use systemd, but that requires
    # version v234 because of https://github.com/systemd/systemd/issues/5629.
    cmd = Command('/usr/sbin/rtcwake -s %i -m "%s"' % (time_s, sleeptype))
    return cmd.run() == 0


