#!/usr/bin/env python
# Copyright (c) 2009 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Authors: Greg Nichols
#          YangKun <ykun@redhat.com>
#          Caspar Zhang <czhang@redhat.com>

import sys, time

from rhcert.suites.hwcert.hwcert.opticalDiskTest import OpticalDiskTest
from rhcert.tags import Constants, DeviceClass
from rhcert.command import Command, HwCertCommandException
from rhcert.compatability import rhcert_get_status_output

class BluRayTest(OpticalDiskTest):

    def __init__(self):
        OpticalDiskTest.__init__(self, "bluray")
        self.mediaTypes = ["bdre",  "bdr", "bd"]
        self.deviceClass = DeviceClass.optical
        self.fileSourceDir = "/usr/bin"   # ranges from 100MB-270MB

    def writer(self, deviceType):
        cdromDevice = self.getLogicalDeviceName()
        """handles BD-R writing a chunk of the filesystem to the media and then checking the
           file sums"""
        print("\n  +++ Start writing ...")
        sys.stdout.flush()

        try:
            # Unmount the drive first in case it's mounted
            Command("umount /dev/%s" % cdromDevice).echoIgnoreErrors()
            time.sleep(10)

            # write the directory to disk
            print("  This may take some time, please be patient ...")
            sys.stdout.flush()
            (status, junk) = rhcert_get_status_output("growisofs -Z /dev/%s -quiet -R %s > /dev/null 2>&1" % (cdromDevice, self.fileSourceDir))
            print(junk)
            if status == 0:
                print("  +++ Write BD-R/RE disk succeeded ! \n")
            else:
                print("  +++ Error: Write BD-R/RE disk failed !\n")
                return False
            self.handleEject(cdromDevice)
            sys.stdout.flush()

            # write done, start to compare
            return self.compare(deviceType, cdromDevice)

        except HwCertCommandException as exception:
            print(exception)
            return False

    def checkMediaSize(self, size):
        if size == 0 :
            print("Error: the bluray disk in drive is blank, test can not continue !\n")
            return False
        # otherwise
        print("  Total file size contained on the bluray disk is : %s KB\n" % size)
        return True

    def run(self):
        """the main function"""
        if not self.initializeDeviceInfo():
            return False

        if self.getMode() == Constants.auto:
            print("Error: The bluray test can not be run in auto mode.")
            return False

        self.getMediaChoice()
        if self.mediaChoice == "bdre":
            description = "good"
            mode = "rewrite"
        elif self.mediaChoice == "bdr":
            description = "new"
            mode = "write"
        elif self.mediaChoice == "bd":
            description = "good"
            mode = "read"
        else:
            print("Error: could not determine media type")
            return False

        self.echoTest(self.getLogicalDeviceName(), self.mediaChoice, description)
        if not self.checkMediaType(self.getLogicalDeviceName(), self.mediaChoice):
            return False

        if self.testDisk(message="%s %s" %(description, self.printMediaName(self.mediaChoice)), readWriteMode=mode, type="bd"):
            return True
        # OTHERWISE
        return False

if __name__ == "__main__":
    test = BluRayTest()
    returnValue = test.do(sys.argv)
    sys.exit(returnValue)
