#!/usr/bin/env python
# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or 
# redistribute it subject to the terms and conditions of the GNU General 
# Public License v.2.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
import os, sys, shutil, re


from rhcert.tags import TestResult
from rhcert.suites.rhcert.platform import KernelTest, CheckSELinux, LogVersions
from rhcert.suites.rhcert.platform import LogDmidecode, CheckHardwareHealth
from rhcert.suites.hwcert.hwcert.procDeviceDetector import ProcDeviceDetector
from rhcert.suites.hwcert.hwcert.hardwareTest import HardwareTest
from rhcert.suites.hwcert.hwcert.deviceclass import DeviceClass

class InfoTest(KernelTest, HardwareTest):

    def __init__(self):
        HardwareTest.__init__(self, "info")
        KernelTest.__init__(self, "hwcert/info")
        self.deviceClass = DeviceClass.system
        self.device = None
        self.source = None

    def logDevices(self, planner):
        devices = list()
        proc = ProcDeviceDetector()
        devices.extend(proc.getDevices(ProcDeviceDetector.procCPUInfo))
        # call the default to log udev devices
        devices.extend(HardwareTest.logDevices(self, planner))
        return devices


    def run(self):

        result = TestResult()
        for subTest in [LogVersions(self),
                        CheckSELinux(self),
                        CheckHardwareHealth(self),
                        LogDmidecode(self)]:
            result.combine(subTest.run())

        result.combine(KernelTest.run(self))

        return result


def unit_test():
    test = InfoTest()
    return test.do(sys.argv)

if __name__ == "__main__":
    import doctest
    doctest.testmod()

    #TODO - commenting out for now because
    #AttributeError: 'InfoTest' object has no attribute 'do'
    #sys.exit(unit_test())
