
import os, string, sys, socket, re, time, decimal, filecmp

from rhcert.tags import Constants, DeviceClass, TestTag
from rhcert.command import Command, HwCertCommandException

from rhcert.suites.hwcert.hwcert.hardwareTest import HardwareTest
from rhcert.suites.hwcert.hwcert.procDeviceDetector import ProcDeviceDetector

class DevicePort(object):
    def __init__(self, deviceName, portNumber, module=None, attributes={}, potentialIPAddresses=[], netDeviceName=""):
        self.deviceName = deviceName
        self.portNumber = portNumber
        self.module = module
        self.attributes = attributes
        self.potentialIPAddresses = potentialIPAddresses
        self.interactive = True
        self.netDeviceName = netDeviceName

class IpDevice():
    def __init__(self, name, ip4="", ip6="", link=""):
        self.name = name
        self.ip4 = ip4
        self.ip6 = ip6
        self.link = link
    def __str__(self):
        return "Device %s -\nip4: %s\nip6: %s\nlink: %s" % (self.name, self.ip4, self.ip6, self.link)

def parseAttributes(valuesString):
    """Parses value string separated by the first : in each line  into dict"""
    listOfLists = [re.split(":", x, 1) for x in re.split("\n", valuesString) if x.strip()]
    attributeDict = {}
    for (key, value) in listOfLists:
        attributeDict[key.strip()] = value.strip()
    return attributeDict

def getAddressFiles(module):
    """Get address files matching with modules.  Returns empty list if none found"""
    try:
        addressFiles = Command("find /sys/devices -iname '*%s*' | xargs -I {} find {} -iname address" % module).getStringList(regex=".*")
    except HwCertCommandException:
        print("Error - Failed to find address file for module %s" % module)
        return []
    return addressFiles

def matchLinkGidIp6(link="", gid="", ip6=""):
    """apply known matching techniques to see if gid is related to device via link or ip6"""
    gid = gid.replace(":", "")
    ip6 = ip6.strip()
    if not (link or ip6) or not gid:
        return False
    gidInLink = linkInGid = gidTransformedInLink = linkInGidTransformed = False
    gidInIp6 = ip6InGid = gidTransformedInIp6 = ip6InGidTransformed = False 
    gidSearchPattern = insert_every(original=gid, cadence=2, inserting=".*")
    #CERTPX-4049 && bug_id=1660786 && bug_id=1664467
    gidTransformed = ""
    try:
        gidTransformed = hex(int(gid[16] + gid[17], 16) ^ 2)[2:] + gid[18:22] + gid[26:]
    except IndexError:
        print("Warning: gid is below expected length")
    gidTransformedPattern = insert_every(original=gidTransformed, cadence=2, inserting=".*")
    if link and re.search("[1-9]", link):
        linkSearchPattern = link.replace(":", "")
        linkSearchPattern = insert_every(original=linkSearchPattern, cadence=2, inserting=".*")
        gidInLink = re.search(gidSearchPattern, link)
        linkInGid = re.search(linkSearchPattern, gid)
        gidTransformedInLink = gidTransformed and re.search(gidTransformedPattern, link)
        linkInGidTransformed = gidTransformed and re.search(linkSearchPattern, gidTransformed)
    if ip6:
        ip6SearchPattern = ip6.replace(":", "")
        ip6SearchPattern = insert_every(original=ip6SearchPattern, cadence=2, inserting=".*")
        gidInIp6 = re.search(gidSearchPattern, ip6)
        ip6InGid = re.search(ip6SearchPattern, gid)
        gidTransformedInIp6 = gidTransformed and re.search(gidTransformedPattern, ip6)
        ip6InGidTransformed = gidTransformed and re.search(ip6SearchPattern, gidTransformed)
    #print("values: \n%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s" % (gidInLink, linkInGid, gidInIp6, ip6InGid,
    #                                                    gidTransformedInLink, linkInGidTransformed,
    #                                                    gidTransformedInIp6, ip6InGidTransformed))
    return gidInLink or linkInGid or gidInIp6 or ip6InGid or gidTransformedInLink \
            or linkInGidTransformed or gidTransformedInIp6 or ip6InGidTransformed

def collectNetDevice(deviceName):
    netDevice = IpDevice(deviceName)
    ipAddrOutput = Command("ip addr show %s" % deviceName).getStringList(regex=".*")

    for item in ipAddrOutput:
        ip4Match = re.match("[ \t]*inet (?P<ipAddress>(\d{1,3}\.){3}\d{1,3})", item)
        ip6Match = re.match("[ \t]*inet6 (?P<address>[a-z0-9:]*)/", item)
        if re.match("[ \t]*link", item):
            netDevice.link = item.strip().split(" ")[1]
        elif ip4Match:
            netDevice.ip4 = ip4Match.group("ipAddress")
        elif ip6Match:
            netDevice.ip6 = ip6Match.group("address")
    return netDevice

def getLinkFromNetDevice(device):
    getLinkString = "ip addr show %s" % device
    outputString = ""
    try:
        outputString = Command(getLinkString).getString(regex="^\s*link.*", singleLine=False)
    except HwCertCommandException:
        print("Error - no ip addr link associated with %s" % device)
        return None
    return outputString

def findNetDeviceFromNetScripts(gid):
    gid = "".join(gid.split(":"))
    try:
        hwaddrs = Command('grep -Re "HWADDR=..:" /etc/sysconfig/network-scripts').getStringDict(regex=".*scripts/ifcfg-(?P<netDevice>.*):HWADDR=(?P<address>.*)", keyGroup="address", valueGroup="netDevice")
    except HwCertCommandException:
        print("Warning: could not determine net device from network-scripts")
        return False
    print(hwaddrs)
    hwaddrs = {"".join(k.split(":")):v for (k,v) in hwaddrs.items()}
    for (address, netDevice) in hwaddrs.items():
        if matchLinkGidIp6(link=address.lower(), gid=gid):
            return netDevice

def getNetDevices(module, gid):
    addressFiles = getAddressFiles(module)
    matchingLinkDevices = []
    if addressFiles:
        netDeviceNames = [ re.search(".*?/net/(?P<netDevice>.*?)/address", x).group("netDevice") for x in addressFiles]
        for deviceName in netDeviceNames:
            netDevice = collectNetDevice(deviceName)
            if matchLinkGidIp6(netDevice.link, gid, netDevice.ip6):
                matchingLinkDevices.append(deviceName)
    anotherDevice = findNetDeviceFromNetScripts(gid)
    if anotherDevice and anotherDevice not in matchingLinkDevices:
        matchingLinkDevices.append(anotherDevice)
    return matchingLinkDevices

def insert_every(original, cadence=2, inserting=""):
    """ Returns new string with <inserting> character(s) placed
        after every <cadence> character of the <original> string """
    def insert_gen():
        count = 0
        for i in range(len(original)):
            if count < cadence:
                count = count + 1
                yield original[i]
            else:
                count = 1
                yield "%s%s" % (inserting, original[i])
    return "".join([x for x in insert_gen()])

def parseDevices(driverModules, ibstatusOutput=None):
    if not ibstatusOutput:
        ibstatusOutput = getIbstatusOutput()
    devicePorts = []
    namePortSplit = re.split("Infiniband device '(.*?)' port (\d+?) status:", ibstatusOutput)
    namePortSplit.pop(0)
    names = namePortSplit[::3]
    ports = namePortSplit[1::3]
    values = namePortSplit[2::3]
    for name, port, valuesString in zip(names, ports, values):
        attributeDict = parseAttributes(valuesString)
        # get module
        # searchString = "|".join(["|".join(x.split("_")) for x in driverModules])
        modules = []
        for mod in driverModules:
            if mod.startswith('vmw_'):
                modules.append(mod)
            else:
                modules.append("|".join(mod.split("_")))
        searchString = "|".join(modules)

        match = re.search(searchString, name)
        module = None
        if match:
            module = match.group(0)

        devicePort = DevicePort(name, port, module, attributeDict)
        # moved this if statement up top.  might just remove next condition..
        if devicePort.module:
            addressFiles = getAddressFiles(devicePort.module)
        elif  attributeDict.get('link_layer') == "InfiniBand":

            try:
                lookupCode = Command("lspci | grep -Ei '(infiniband|fabric)'").getString(regex="^(?P<devcode>.*?)\s", regexGroup="devcode")
            except HwCertCommandException:
                print("Device module not found in ibstatus.")
                print("Case insensitive words infiniband and fabric not found in lspci")
                continue
            try:
                addressFiles = Command("find /sys/ -iname address | grep %s" % lookupCode).getStringList(regex=".*")
            except HwCertCommandException:
                print("Lookup code %s found, but there was no address file in /sys/")
                continue
        else:
            continue
        potentialIPAddresses = []
        for addressFile in addressFiles:
            try:
                address = Command("cat %s" % addressFile).getString()
            except HwCertCommandException:
                print("Warning - Failed to get address file contents for %s" % addressFile)
                continue
            ipAddrCommandString = "ip addr | grep -A 1 %s" % address
            try:
                ipAddress = Command(ipAddrCommandString).getString(regex=".*inet (?P<ipAddress>(\d{1,3}\.){3}\d{1,3})", regexGroup ="ipAddress", singleLine=False)
            except:
                continue
            potentialIPAddresses.append(ipAddress)
        if not potentialIPAddresses:
            print("Warning - failed to get ipv4 addresses from ip addr lines")
            print("Using default gid for the device/port reported in ibstatus to find ip address")
            gid = devicePort.attributes.get('default gid')
            print("gid found: %s" % gid)

            netDevices = collectIpDevices()

            try:
                for netDevice in netDevices:
                    if matchLinkGidIp6(netDevice.link, gid, netDevice.ip6):
                        potentialIPAddresses.append(netDevice.ip4)
            except:
                print("Warning - could not calculate ipv4 address using default gid either")

        devicePort.potentialIPAddresses = potentialIPAddresses
        devicePorts.append(devicePort)
    return devicePorts

def collectIpDevices():

    ipAddrOutput = Command("ip addr").getStringList(regex=".*")
    netDevices = []

    debugString = ""
    deviceStrings = []
    try:
        results = []
        ipAddrOutput = Command("ip addr").getStringList(regex=".*")
        netDevices = []

        for item in ipAddrOutput:
            ip4Match = re.match("[ \t]*inet (?P<ipAddress>(\d{1,3}\.){3}\d{1,3})", item)
            ip6Match = re.match("[ \t]*inet6 (?P<address>[a-z0-9:]*)/", item)
            if re.match("\d", item):
                name = item.strip().split(":")[1]
                name = name.strip()
                netDevices.append(IpDevice(name))
            elif re.match("[ \t]*link", item):
                link = item.strip().split(" ")[1]
                netDevices[-1].link = link
            elif ip4Match:
                ip4 = ip4Match.group("ipAddress")
                netDevices[-1].ip4 = ip4
            elif ip6Match:
                ip6 = ip6Match.group("address")
                netDevices[-1].ip6 = ip6
    except HwCertCommandException:
        print("Error collecting ip devices")
    return netDevices

def getIbstatusOutput():
    try:
        stringList = Command("ibstatus").getIgnoreErrors()
    except HwCertCommandException:
        print("Error - Failed to run ibstatus")
        return ""
    if stringList:
        return '\n'.join(stringList)
    else:
        return ""
