#!/usr/bin/env python
# Copyright (c) 2008 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or 
# redistribute it subject to the terms and conditions of the GNU General 
# Public License v.2.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols <gnichols@redhat.com>

import sys, time

from rhcert.tags import Constants, DeviceClass, TestTag
from rhcert.command import Command
from rhcert.environment import Environment
from rhcert.continuation import Continuation

from rhcert.suites.hwcert.hwcert.hardwareTest import HardwareTest

class RebootTest(HardwareTest):

    def __init__(self):
        HardwareTest.__init__(self, "reboot")
        self.interactive = False # will prompt in --mode normal, but runs ok in --mode auto
        self.deviceClass = DeviceClass.system
        self.environment = Environment()
        self.continuation = Continuation(self.environment)
        self.rebootTimeLimit = self.environment.getRebootTimelimit()
        self.priority = 1001 # run last
        self.coreCollector = "makedumpfile -d 31"
        self.kdumpConfigPath = "/etc/kdump.conf"

    def tags(self):
        return [TestTag.noninteractive, TestTag.certification]

    def plan(self, planner):
        tests = list()
        # manual planning only
        return tests

    def add(self, planner):
        pass

    def reboot(self):
        self.markOutput("initialize")
        print("The system must be restarted for this test")

        if self.getMode() != Constants.auto:
            if not self.promptConfirm("Ready to restart?"):
                return False

        # set up restart, and log start time
        self.continuation.setInitConfig(self.getPath(), Constants.reboot)

        # we need a delay here to give hwcert time to finish writing results.xml
        self.closeOutput()
        sys.stdout.flush()
        self.waitForLull()

        Command("shutdown -r 0").echo(ignoreErrors=True)

        # wait here for reboot
        waitTime = 60 #sec
        print("Waiting for shutdown...")
        time.sleep(waitTime)
        print("Error: shutdown took too long") 
        return False

    def run(self):

        if self.incomplete and self.continuation.isInitialized():
            self.markOutput("verify")
            result = self.continuation.verify(marker=self.getPath())
            self.continuation.removeInitConfig()
            self.closeOutput()
        else:
            result = self.reboot()

        return result

if __name__ == "__main__":
    test = RebootTest()
    returnValue = test.do(sys.argv)
    sys.exit(returnValue)
