#!/usr/bin/env python
# Copyright (c) 2017 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or 
# redistribute it subject to the terms and conditions of the GNU General 
# Public License v.2.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Pooja Tiwari
#
import sys
from rhcert.suites.rhcert.platform import GenerateSystemReport
from rhcert.tags import TestTag, Constants
from rhcert.environment import Environment
from rhcert.suites.hwcert.hwcert.procDeviceDetector import ProcDeviceDetector
from rhcert.suites.hwcert.hwcert.hardwareTest import HardwareTest
from rhcert.suites.hwcert.hwcert.deviceclass import DeviceClass

class SosReportTest(HardwareTest):

    def __init__(self):
        HardwareTest.__init__(self, "sosreport")
        self.environment = Environment()
        self.mandatory = Constants.post
        self.priority = 1001
        self.deviceClass = DeviceClass.system
        self.device = None
        self.source = None

    def tags(self):
        return [TestTag.noninteractive, TestTag.certification]

    def plan(self, planner):
        tests = list()
        tests.append(self.makeCopy())
        return tests

    def logDevices(self, planner):
        devices = list()
        proc = ProcDeviceDetector()
        devices.extend(proc.getDevices(ProcDeviceDetector.procCPUInfo))
        # call the default to log udev devices
        devices.extend(HardwareTest.logDevices(self, planner))
        return devices


    def run(self):
        return GenerateSystemReport(self).run()

def unit_test():
    test = SosReportTest()
    return test.do(sys.argv)

if __name__ == "__main__":
    import doctest
    doctest.testmod()
